package net.ko.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import net.ko.displays.Display;
import net.ko.displays.KObjectDisplay;
import net.ko.framework.Ko;
import net.ko.kobject.KObject;
import net.ko.utils.KProperties;
import net.ko.utils.KStrings;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class KObjectController {
	private HashMap<String,KObjectFieldController> members;
	private String koClass;
	private String caption;
	private String display;
	protected KProperties messages;
	protected KProperties er;
	protected KTransformer transformer;
	
	public void setMessages(KProperties messages) {
		if(this.messages!=messages){
			for(Map.Entry<String,KObjectFieldController> entry:members.entrySet())
				entry.getValue().setMessages(messages);
			this.messages = messages;
		}
		
	}
	public void setEr(KProperties er) {
		if(this.er!=er){
			for(Map.Entry<String,KObjectFieldController> entry:members.entrySet())
				entry.getValue().setEr(er);
			this.er = er;
		}
	}	
	public KObjectController(String koClass){
		this.members=new HashMap<String, KObjectFieldController>();
		this.koClass=koClass;
	}
	public String getClassName(){
		return this.koClass;
	}	
	public void initFromXML(Node classXMLElement){
		String memberCaption=((Element)classXMLElement).getAttribute("caption");
		if(memberCaption!=null)
			caption=memberCaption;
		String memberDisplay=((Element)classXMLElement).getAttribute("display");
		String strTransformer=((Element)classXMLElement).getAttribute("transformer");
		setTransformer(strTransformer);
		if(memberDisplay!=null)
			display=memberDisplay;
		NodeList map=classXMLElement.getChildNodes();
		for(int i=0;i<map.getLength();i++){
			if(map.item(i).getNodeName().equalsIgnoreCase("member")){
				Element e=(Element)map.item(i);
				String memberName=e.getAttribute("name");
				KObjectFieldController kofc=new KObjectFieldController(memberName);
				members.put(memberName, kofc);
				kofc.initFromXml(e);
				kofc.setClassName(koClass);
				kofc.setPos(i);
				kofc.setTransformer(transformer);
			}
		}
	}
	public HashMap<String, KObjectFieldController> getMembers(){
		return members;
	} 
	public KObjectFieldController getFieldController(String memberName){
		KObjectFieldController koCtrl=null;
		if(members.containsKey(memberName)){
			koCtrl=members.get(memberName);
		}else
		{
			koCtrl=new KObjectFieldController(memberName);
			koCtrl.setClassName(koClass);
		}
		return koCtrl;
	}
	public String toJSON(){
		String result="{"+KStrings.implode(",",members.values().toArray())+"}";
		return result;
	}
	public boolean isValid(KObject ko){
		boolean result=true;
		for(Map.Entry<String,KObjectFieldController> entry:members.entrySet())
			try {
				result=result && entry.getValue().isValid(ko.getAttribute(entry.getKey())+"");
			} catch (Exception e) {}
		return result;
	}
	public HashMap<String, ArrayList<String>> messages(KObject ko){
		HashMap<String, ArrayList<String>> result=new HashMap<String, ArrayList<String>>();
		for(Map.Entry<String,KObjectFieldController> entry:members.entrySet())
			try {
				result.put(entry.getKey(), entry.getValue().messages(ko.getAttribute(entry.getKey())+""));
			} catch (Exception e) {}
		return result;		
	}
	public String toString(){
		String result="";
		for(Map.Entry<String, KObjectFieldController> e:members.entrySet()){
			result+=e.getKey()+" : "+e.getValue().toString()+Display.getDefault().getEndLigne();
		}
		return result;
	}
	public String getCaption() {
		return caption;
	}
	public void setCaption(String caption) {
		this.caption = caption;
	}
	public String getDisplay() {
		return display;
	}
	public void setDisplay(String display) {
		this.display = display;
	}
	@SuppressWarnings("unchecked")
	public KObjectDisplay getDisplayInstance(){
		KObjectDisplay koDisp=null;
		if(display!=null&&!"".equals(display)){
			try {
				Class<KObjectDisplay> dispClass= (Class<KObjectDisplay>) Class.forName(display);
				koDisp=dispClass.newInstance();
			} catch (ClassNotFoundException | InstantiationException | IllegalAccessException e) {
				koDisp=Ko.defaultKoDisplay();
			}
		}else
			koDisp=Ko.defaultKoDisplay();
		return koDisp;
	}
	@SuppressWarnings("unchecked")
	public void setTransformer(String strTransformer){
		if(strTransformer!=null&&!"".equals(strTransformer)){
			try {
				Class<KTransformer> transformerClass= (Class<KTransformer>) Class.forName(strTransformer);
				transformer=transformerClass.newInstance();
			} catch (ClassNotFoundException | InstantiationException | IllegalAccessException e) {
				transformer=Ko.ktransformer();
			}
		}else
			transformer=Ko.ktransformer();
	}
}
