/**
 * Classe KClassCreator
 * 
 * @author jcheron
 * @link http://www.kobject.net/
 * @copyright Copyright kobject 2008-2010
 * @license http://www.kobject.net/index.php?option=com_content&view=article&id=50&Itemid=2  BSD License
 * @version $Id: KClassCreator.java,v 1.7 2011/01/14 01:12:55 jcheron Exp $
 * @package ko.ksql
 */
package net.ko.creator;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Element;

import net.ko.events.EventFileListener;
import net.ko.ksql.KDataBase;
import net.ko.ksql.KDbField;
import net.ko.utils.KApplication;
import net.ko.utils.KString;
import net.ko.utils.KStrings;
import net.ko.utils.KTemplateFile;


public class KClassCreator {
	private KDataBase db;
	private String tableName;
	private String packageName;
	private String className;
	private HashMap<String,Object> fields;
	private KTemplateFile tplFile;
	private String templateFolder;
	private HashMap<String, Object> fieldMap;
	
	private EventFileListener eventFileListener;
	
	public void addFileListener(EventFileListener eventFileListener){
		this.eventFileListener=eventFileListener;
	}
	
	public String getTemplateFolder() {
		return templateFolder;
	}

	public void setTemplateFolder(String templateFolder) {
		this.templateFolder = templateFolder;
		tplFile.open(getPathName(this.templateFolder+"/class.tpl"));
	}
	
	private String getPathName(String name){
		String result="";
		String rootPath=KApplication.getRootPath(KClassCreator.class);
		if(!rootPath.endsWith(".jar"))
			result=rootPath+"/"+name;
		else
			result=name;
		return result;
	}

	public void setClassName(String className) {
		this.className = className;
	}

	public KDataBase getDb() {
		return db;
	}

	public String getTableName() {
		return tableName;
	}

	@Override
	public String toString() {
		return tplFile.toString();
	}

	public void setTableName(String tableName) {
		this.tableName = tableName;
	}

	public void setDb(KDataBase db) {
		this.db = db;
	}
	public KClassCreator(KDataBase db,String tableName,String templateFolder) {
		super();
		this.db = db;
		this.tableName=tableName;
		setFields();
		tplFile=new KTemplateFile();
		setTemplateFolder(templateFolder);
	}
	public KClassCreator(KDataBase db,String tableName) {
		this(db,tableName,"net/ko/templates/java");
	}

	private void setFields(){
		try {
			fields=db.getFieldNamesAndTypes(tableName);
			fields.remove("id");
			fields=correctFields(fields);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	public String mkMembers(){
		return showWithMask("member.tpl", "\n");
	}
	public String mkGetters(){
		return showWithMask("get.tpl", "\n");		
	}
	public String mkSetters(){
		return showWithMask("set.tpl", "\n");
	}
	public String mkGettersAndSetters(){
		return mkGetters()+mkSetters();
	}
	public String mkToString(){
		return "return "+showWithMask("tostring.tpl", "+",true);
	}
	public String getClassName(){
		if (className==null)
			return "K"+KString.capitalizeFirstLetter(tableName.replace(" ", "_"));
		else
			return className;
	}
	private KTemplateFile mkTpl(){
		KTemplateFile tpl=tplFile.clone();
		tpl.parseWith("package", "kernel."+getDb().getBaseName());
		tpl.parseWith("className", getClassName());
		parseTpl(tpl);	
		return tpl;
	}
	public String mkClass(){
		tplFile=mkTpl();
		return mkTpl().toString();
	}
	private String parseTpl(KTemplateFile aTpl){
		aTpl.parseWith("tableName", tableName);
		aTpl.parseWith("members", mkMembers());
		aTpl.parseWith("setters", mkSetters());
		aTpl.parseWith("getters", mkGetters());
		aTpl.parseWith("toString", mkToString());
		aTpl.parseWith("fieldMap", mkFieldMap());
		try {
			aTpl.parseWith("keyFields", new KStrings(db.getKeyFields(tableName)).implode(","));
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return aTpl.toString();
	}
	public String mkClassMembers(){
		return parseTpl(tplFile);
	}
	private HashMap<String, Object> correctFields(HashMap<String, Object> fields){
		HashMap<String, Object> ret=new HashMap<String, Object>();
		fieldMap=new HashMap<String, Object>();
		for (Map.Entry<String, Object> f : fields.entrySet()){
			String newVal=f.getKey().replace(" ", "_");
			if(!newVal.equals(f.getKey()))
				fieldMap.put(newVal,f.getKey());
			ret.put(newVal, f.getValue());
		}
		return ret;
	}
	private String mkFieldMap(){
		return showWithMask(fieldMap, "fieldmap.tpl", "\n");
	}
	private String showWithMask(HashMap<String, Object> map,String templateFile,String glue){
		KStrings ks=null;
		ks = new KStrings(map);
		KTemplateFile tpl=new KTemplateFile();
		tpl.open(getPathName(templateFolder+"/"+templateFile));
		String sTpl=tpl.getText();
		return ks.showWithMask(glue, sTpl);
	}
	private String showWithMask(String templateFile,String glue,boolean onlyPrimitive){
		if(onlyPrimitive){
			HashMap<String, Object> nFields=new HashMap<>();
			for(Map.Entry<String, Object> e:fields.entrySet()){
				if(e.getValue() instanceof KDbField)
					nFields.put(e.getKey(), e.getValue());
			}
			return showWithMask(nFields, templateFile, glue);
		}else
			return showWithMask(fields, templateFile, glue);
	}
	private String showWithMask(String templateFile,String glue){
		return showWithMask(templateFile, glue,false);
	}
	public void saveAs(String packageName){
		saveAs(getPathName(""), packageName);
	}
	public void saveAs(String path,String packageName){
		this.packageName=packageName;
		save(path);
	}
	public void save(){
		save(getPathName(""));
	}
	public void save(String path){
		if(eventFileListener!=null)
			tplFile.addFileListener(eventFileListener);
		tplFile.saveAs(path, packageName, getClassName());		
	}
	public boolean existMember(String member){
		return fields.containsKey(member);
	}
	private void addConstraint(String cName,String fkClassName){
		KTemplateFile tpl=new KTemplateFile();
		tpl.open(getPathName(templateFolder+"/"+cName+".tpl"));
		tpl.parseWith("fkClassName", fkClassName);
		tplFile.parseWith("constraints", tpl.getText(), true);		
	}
	public void addBelongsTo(String fkClassName){
		addConstraint("belongsto", fkClassName);
	}
	public void addHasMany(String fkClassName){
		addConstraint("hasmany", fkClassName);
	}
	public void addFkMember(String fkTableName){
		fields.put(fkTableName, "K"+KString.capitalizeFirstLetter(fkTableName));
	}

	public void addManyMember(String fkTableName){
		fields.put(fkTableName+"s", "KListObject<"+"K"+KString.capitalizeFirstLetter(fkTableName)+">");
	}
	public void clean(){
		tplFile.clean();
	}
	public void mkController(KDataBase db,KControllerCreator cCreator,Element xmlElement){
		setFields();
		for(Map.Entry<String, Object> entryFields:fields.entrySet()){
			KDbField field=(KDbField)entryFields.getValue();
			cCreator.createMemberElement(xmlElement, field);
		}
	}	
}
