package net.ko.framework;

import java.io.IOException;
import java.sql.SQLException;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import net.ko.displays.KHttpDisplay;
import net.ko.http.objects.KHttpSession;
import net.ko.mapping.KAjaxIncludes;
import net.ko.mapping.KXmlMappings;
import net.ko.types.KRestartType;

/**
 * Classe Singleton de lancement d'une application Web java de type KObject
 * @author jcheron
 *
 */
public class KoHttp extends Ko {
	private KAjaxIncludes ajaxIncludes;

	/**
	 * @return l'instance d'application KoHttp
	 */
	public static boolean allowAjaxIncludes=true;
	
	public static KoHttp getInstance(){
		if(instance==null){
			synchronized (KoHttp.class) {
			instance=new KoHttp();
			}
		}
		try{
			instance= (KoHttp) instance;
		}catch(ClassCastException ce){
			synchronized (KoHttp.class) {
			instance=new KoHttp();
			}
		}
		return (KoHttp) instance;
	}
	
	/**
	 * Démarre l'application Ko, en créant si nécessaire l'instance d'application active KoHttp
	 * @param session session http en cours
	 */	
	public static void krestart(HttpSession session) throws ClassNotFoundException, SQLException, IOException, InstantiationException, IllegalAccessException{
		Ko.kstop();
		kstart(session, true);
	}
	
	public static void krestart(HttpSession session,String options) throws ClassNotFoundException, SQLException, IOException, InstantiationException, IllegalAccessException{
		if(options==null||"".equals(options.replace(";", "")))
			krestart(session);
		else{
			String[] optionsValues=options.split(";");
			for(String option:optionsValues){
				if(option.equals(KRestartType.rtBddConn.value))
					kdatabase();
				if(option.equals(KRestartType.rtValidation.value))
					Ko.getInstance().setValidationFile();
				if(option.equals(KRestartType.rtMessages.value)){
					Ko.getInstance().setMessagesFile();
					Ko.getInstance().setErFile();
				}
				if(option.equals(KRestartType.rtMappings.value)){
					KXmlMappings.stop();
					Ko.getInstance().setMappingFile();
				}
				if(option.equals(KRestartType.rtCssVars.value)){
					Ko.styleSheets=new HashMap<>();
					Ko.getInstance().loadCssVars();
				}
			}
		}
	}	
	public static void kstart(HttpSession session) {
		kstart(session, true);
	}
	
	public static void kstart(HttpSession session,boolean noDb) {
		new KHttpDisplay();
		KoHttp inst=(KoHttp)getInstance(session);
		if(!inst.isLoaded()){
			inst.setKsession(new KHttpSession(session));
			inst.start(noDb);
			inst.setLoaded(true);
		}
		//inst.init();
	}
	
	private static KoHttp getInstance(HttpSession session) {
		return getInstance();
	}
	public static void setSession(HttpSession session){
		((KHttpSession)KoHttp.ksession()).setHttpSession(session);
	}
	
	public static void setPath(String koPath){
		getInstance().setKoPath(koPath);
	}
	
	public static void kstartFromController() throws ClassNotFoundException, SQLException, IOException, InstantiationException, IllegalAccessException{
		KoHttp.kstart(null);
	}
	
	public static void kstartFromController(String koPath) throws ClassNotFoundException, SQLException, IOException, InstantiationException, IllegalAccessException{
		getInstance().setKoPath(koPath);
		KoHttp.kstart(null);
	}
	
	public static HttpSession getSession(){
		return ((KHttpSession)KoHttp.ksession()).getHttpSession();
	}
	
	public KAjaxIncludes getAjaxIncludes() {
		return ajaxIncludes;
	}
	public static KAjaxIncludes kajaxIncludes(){
		KoHttp ko=(KoHttp)getInstance();
		return ko.getAjaxIncludes();
	}
	
	public static String kajaxIncludes(HttpServletRequest request,HttpServletResponse response){
		KoHttp ko=(KoHttp)getInstance();
		KAjaxIncludes ajaxIncludes=ko.getAjaxIncludes();
		ajaxIncludes.process(request, response);
		return ajaxIncludes.toString();
	}
	public static String kajaxIncludes(HttpServletRequest request){
		String result="";
		if(request.getAttribute("ajaxIncludes")!=null)
			result=request.getAttribute("ajaxIncludes")+"";
		return result;
	}
	@Override
	public KXmlMappings setMappingFile(String mappingFile) {
		KXmlMappings result=super.setMappingFile(mappingFile);
		ajaxIncludes=result.getAjaxIncludes();
		return result;
	}

}
