package net.ko.http.views;

import java.io.File;

import javax.servlet.http.HttpServletRequest;

import net.ko.http.objects.KQueryString;
import net.ko.http.objects.KRequest;

/**
 * Barre de navigation visuelle en HTML permettant de se déplacer 
 * entre les pages gérées avec un objet KPagination
 * @see net.ko.http.views.KPagination
 * @author jcheron
 *
 */
public class KNavBarre {
	public String folder="images/navBarre";//Dossier contenant les images de la barre
	public String activePageStyle="select";//Style css du numéro de la page active
	public int pageNavCount=5;//Nombre de pages à afficher dans la barre
	public int lastPage;//Numéro de la dernière page
	private int activePage=1;//Numéro de la page active
	private int rowCount;
	private HttpServletRequest request;
	private String requestUrl="";
	private boolean ajax=false;
	private String ajaxDivContentRefresh="ajxContent";
	
	private String getButton(String img,String pDefault){
		String realPath=request.getSession().getServletContext().getRealPath(img);
		File f=new File(realPath);
		if (f.exists()){
			return "<img src='"+img+"' align='absmiddle' border=0>";
		}
		else
		{return pDefault;}
	}
	public String cmdFirst(){
		return cmdFirst("first.gif");
	}
	public String cmdFirst(String imgName){
		return this.getButton(this.folder+"/"+imgName,"<div class='navBarreBtn'>|<div class='arrow-left'></div>&nbsp;</div>");
	}
	public String cmdLast(){
		return cmdLast("last.gif");
	}
	public String cmdLast(String imgName){
		return this.getButton(this.folder+"/"+imgName,"<div class='navBarreBtn'>&nbsp;<div class='arrow-right'></div>|</div>");
	}
	public String cmdPrevious(){
		return cmdPrevious("previous.gif");
	}
	public String cmdPrevious(String imgName){
		return this.getButton(this.folder+"/"+imgName,"<div class='navBarreBtn'>&nbsp;<div class='arrow-left'></div>&nbsp;</div>");
	}
	public String cmdNext(){
		return cmdNext("next.gif");
	}
	public String cmdNext(String imgName){
		return this.getButton(this.folder+"/"+imgName,"<div class='navBarreBtn'>&nbsp;<div class='arrow-right'></div>&nbsp;</div>");
	}
	private int getPageD(){
		int div=Math.round(this.activePage-this.pageNavCount/2);
		if (div<1) div=1;
		return div;
	}
	public KNavBarre(HttpServletRequest request){
		this(request,"");
	}
	public KNavBarre(HttpServletRequest request,String folder) {
		this.request=request;
		if (!"".equals(folder)) this.folder=folder;
	}
	public void gotoPage(int aPage){
		this.activePage=aPage;
	}
	public String toString(){
		int previous=this.activePage-1;
		if (previous<1) previous=1;
		int next=this.activePage+1;
		if (next>this.lastPage) next=this.lastPage;
		int pageD=this.getPageD();
		int pageF=pageD+this.pageNavCount;
		if (pageF>this.lastPage) pageF=this.lastPage;
		if (pageF-pageD<this.pageNavCount) pageD=pageF-this.pageNavCount;
		if (pageD<1) pageD=1;
		int i=pageD;
		String result="<div class='navBarre'><table border=0 cellPadding=0 cellSpacing=0><tr>";
		result+="\n<td>"+getLink("Atteindre la première page", KQueryString.setValue(this.getQueryString(),"page", "1"), this.cmdFirst())+"</td>";
		result+="\n<td>"+getLink("Atteindre la page n°"+previous, KQueryString.setValue(this.getQueryString(),"page", String.valueOf(previous)), this.cmdPrevious())+"</td>";
		while (i<=pageF) {
			String style="";
			if (i==this.activePage) style=" class='"+this.activePageStyle+"' ";
			result+="\n<td align='center' valign='middle'"+style+">";
			result+="&nbsp;"+getLink("Atteindre la page "+i, KQueryString.setValue(this.getQueryString(),"page", String.valueOf(i)), i+"")+"&nbsp;</td>";
			i++;
		}
		
		result+="\n<td>"+getLink("Atteindre la page n°"+next, KQueryString.setValue(this.getQueryString(),"page", String.valueOf(next)), this.cmdNext())+"</td>";
		result+="\n<td>"+getLink("Atteindre la dernière page", KQueryString.setValue(this.getQueryString(),"page", String.valueOf(lastPage)), this.cmdLast())+"</td>";
		result+="\n</tr></table></div>";
		return result;
	}
	private String getLink(String title,String queryString,String caption){
		String result="";
		if(ajax)
			result="<a title='"+title+"' onclick=\"Forms.Utils.get('"+this.ajaxDivContentRefresh+"','"+this.getRequestUrl()+"','_refresh&"+queryString+"');\">"+caption+"</a>";
		else
			result="<a title='"+title+"' href='"+getRequestUrl()+"?"+queryString+"'>"+caption+"</a>";
		return result;
	}
	public String getRequestUrl() {
		if ("".equals(requestUrl) || requestUrl==null)
			requestUrl=request.getRequestURI();
		return requestUrl;
	}
	public void setRequestUrl(String requestUrl) {
		this.requestUrl = requestUrl;
	}
	public boolean isAjax() {
		return ajax;
	}
	public void setAjax(boolean ajax) {
		this.ajax = ajax;
	}
	public String getAjaxDivContentRefresh() {
		return ajaxDivContentRefresh;
	}
	public void setAjaxDivContentRefresh(String ajaxDivContentRefresh) {
		this.ajaxDivContentRefresh = ajaxDivContentRefresh;
	}
	public int getRowCount() {
		return rowCount;
	}
	public void setRowCount(int rowCount) {
		this.rowCount = rowCount;
	}
	public String getQueryString(){
		String result=request.getQueryString();
		result=KQueryString.setValue(result, "_nb", rowCount+"");
		return result;
	}
}
