/**
 * Classe KConstraint
 * 
 * @author jcheron
 * @link http://www.kobject.net/
 * @copyright Copyright kobject 2008-2010
 * @license http://www.kobject.net/index.php?option=com_content&view=article&id=50&Itemid=2  BSD License
 * @version $Id: KConstraint.java,v 1.4 2011/01/21 01:22:52 jcheron Exp $
 * @package ko.kobject
 */
package net.ko.kobject;

import java.io.Serializable;

import net.ko.framework.KDebugConsole;
import net.ko.ksql.KDataBase;
import net.ko.list.MapArrayString;


public abstract class KConstraint implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = -7154829855700471488L;
	protected String where;
	protected Class<? extends KObject> clazz;
	protected KObject owner;
	protected String member;
	protected boolean lazy;
	protected int depth;
	protected boolean autoUpdate;
	
	public KObject getOwner() {
		return owner;
	}
	public void setOwner(KObject owner) {
		this.owner = owner;
	}
	public KConstraint(String member,Class<? extends KObject> clazz ,String where) {
		super();
		this.where = where;
		this.member=member;
		this.clazz=clazz;
		this.lazy=false;
		depth=-1;
		autoUpdate=false;
	}
	public String getWhere() {
		return where;
	}
	public void setWhere(String where) {
		this.where = where;
	}
	public Class<? extends KObject> getClazz() {
		return clazz;
	}
	
	public Class<? extends KObject> getClazzList() {
		return clazz;
	}
	
	public void setClazz(Class<? extends KObject> clazz) {
		this.clazz = clazz;
	}
	public void load(KDataBase db){
		load(db, new KDuoClasseList());
	}
	public void load(KDataBase db,KDuoClasseList duoClasseList){
		KDebugConsole.print(this.clazz+"", "KOBJECT", this.getClass().getSimpleName()+".load");
		duoClasseList.add(owner.getClass(), this.clazz);
	}
	public void load(KListObject<? extends KObject> kl){
		KDebugConsole.print(this.clazz+"", "KOBJECT", this.getClass().getSimpleName()+".loadByKL");
	}

	public void preload(MapArrayString<KConstraint> membersKeys) throws SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException{
		membersKeys.put(this, getPkFieldValue()+"");
	}
	public abstract void save(KDataBase db);
	public String getMember() {
		return member;
	}
	public KDuoClasse getDuoClasse(){
		return new KDuoClasse(owner.getClass(), this.clazz);
	}
	public boolean isMultiple(){
		return !getClass().equals(KConstraintBelongsTo.class);
	}
	public abstract String getSql() throws InstantiationException, IllegalAccessException ;
	public String getFkField(){
		String result="";
		if(where!=null & !"".equals(where))
			result=(where.substring(0, where.indexOf("="))).trim();
		return result;
	}
	public String getPkField(){
		String result="";
		if(where!=null & !"".equals(where))
			result=(where.substring(where.indexOf("{")+1, where.indexOf("}"))).trim();
		return result;
	}
	public Object getPkFieldValue() throws SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException{
		Object result="";
		if(owner!=null)
			result=owner.getAttribute(getPkField(), true);
		return result;
	}
	public Object getFkFieldValue() throws SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException{
		Object result="";
		if(owner!=null)
			result=owner.getAttribute(getFkField(), true);
		return result;
	}
	public boolean isLazy() {
		return lazy;
	}
	public KConstraint setLazy(boolean lazy) {
		this.lazy = lazy;
		return this;
	}
	
	@Override
	public String toString() {
		return member;
	}
	public void update(){
		
	}
	@Override
	public boolean equals(Object obj) {
		boolean result=false;
		if(obj instanceof KConstraint)
			result=clazz.equals(((KConstraint) obj).getClazz())&&member.equals(((KConstraint) obj).getMember());
		return result;
	}
	@Override
	public int hashCode() {
		return (clazz+member).hashCode();
	}
	public int getDepth() {
		return depth;
	}
	public KConstraint setDepth(int depth) {
		this.depth = depth;
		return this;
	}
	public boolean isAutoUpdate() {
		return autoUpdate;
	}
	public KConstraint setAutoUpdate(boolean autoUpdate) {
		this.autoUpdate = autoUpdate;
		return this;
	}
}
