/**
 * Classe KlistObject
 * 
 * @author jcheron
 * @link http://www.kobject.net/
 * @copyright Copyright kobject 2008-2012
 * @license http://www.kobject.net/index.php?option=com_content&view=article&id=50&Itemid=2  LGPL License
 * @version $Id: KListObject.java,v 1.7 2011/01/21 01:22:52 jcheron Exp $
 * @package ko.kobject
 */

package net.ko.kobject;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.logging.Level;

import net.ko.cache.KCache;
import net.ko.displays.Display;
import net.ko.events.KEventType;
import net.ko.events.KEvents;
import net.ko.events.KFireEvent;
import net.ko.framework.KDebugConsole;
import net.ko.framework.Ko;
import net.ko.interfaces.IKommande;
import net.ko.ksql.KDataBase;
import net.ko.ksql.KDbResultSet;
import net.ko.ksql.KSqlQuery;
import net.ko.list.MapArrayString;
import net.ko.utils.KHash;
import net.ko.utils.KQuickSort;
import net.ko.utils.KReflection;
import net.ko.utils.KStrings;
import net.ko.utils.KStrings.KGlueMode;


/**
 * Classe générique permettant de gérer une liste d'instances de KObject
 * Collection d'objet permettant la sérialisation avec une base de données
 * @author jcheron
 *
 * @param <T>
 */
public class KListObject<T extends KObject> implements Iterable<T>,Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 7610568066594751221L;
	public Class<T> getClazz() {
		return clazz;
	}
	protected ArrayList<T> items;
	private ArrayList<T> deletedItems;
    private final Class<T> clazz;
	private String sql;
	@SuppressWarnings("rawtypes")
	private static Field getField(Class clazz, String fieldName) throws NoSuchFieldException {
	    try {
	      return clazz.getDeclaredField(fieldName);
	    } catch (NoSuchFieldException e) {
	      Class superClass = clazz.getSuperclass();
	      if (superClass == null) {
	        throw e;
	      } else {
	        return getField(superClass, fieldName);
	      }
	    }
	}	
	@SuppressWarnings({ "unused", "rawtypes" })
	private String getGenericType(){
		//
		String ret="";
		try {
			ret=getField(this.getClass(),"items").getGenericType().toString();
			Type type = getField(this.getClass(),"items").getGenericType();
		 	ParameterizedType pType = (ParameterizedType) type;
		 	Type[] types = pType.getActualTypeArguments();
		 	Class genC = (Class) types[0];
		 	ret=genC.getName();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return ret;
	}
	public KListObject(Class<T> clazz){
		this.clazz=clazz;
		items=new ArrayList<T>();
		deletedItems=new ArrayList<T>();
		setSql("");
	}
    public String getClassName() {
		return clazz.getName();
	}

	public T get(int index){
		return items.get(index);
	}
	@Override
	public String toString() {
		String ret="";
		for (T unT : items) {
			ret+=unT.toString()+Display.getDefault().getEndLigne();
		}
		return ret;
	}
	public String invoke(String memberName) {
		String ret="";
		for (T unT : items) {
			Object o=KReflection.getMemberValue(memberName, unT);
			if(o!=null)
				ret+=o+Display.getDefault().getEndLigne();
			else
				ret+=Display.getDefault().getEndLigne();
		}
		return ret;
	}

	public void add(KObject ko){
		add(ko,false);
	}
	@SuppressWarnings("unchecked")
	public void add(KObject ko,boolean silent){
		if (!silent)
			ko.toAdd();
		items.add((T) ko);
	}
	
	@SuppressWarnings("unchecked")	
	public void add(int index,KObject ko){
		ko.toAdd();
		items.add(index,(T) ko);
	}
	
	/**
	 * Ajoute une liste à la liste courante
	 * @param alistObject KListObject à ajouter
	 */
	public void addAll(KListObject<T> alistObject){
		items.addAll(alistObject.getItems());
	}

	@SuppressWarnings("unchecked")	
	public void set(int index,KObject ko){
		items.set(index,(T) ko);
	}
	
	@SuppressWarnings("unchecked")		
	public void swap(int i, int j){
		   KObject temporaire;
		   temporaire =items.get(i);
		   items.set(i, items.get(j));
		   items.set(j,(T) temporaire);		
	}
	
	public boolean delete(int index){
		return remove(items.get(index));
	}
	public void deleteByKeys(String keysValues,String sep){
		deleteByKeys(keysValues, sep, "");
	}
	public void deleteByKeys(String keysValues,String sep,String preserveKeys){
		ArrayList<KObject> toDelete=new ArrayList<>();
		for(KObject klo:items){
			if("".equals(preserveKeys)){
			if((sep+keysValues+sep).contains(sep+klo.getFirstKeyValue()+sep))
				toDelete.add(klo);
			}else{
				if(!(sep+preserveKeys+sep).contains(sep+klo.getFirstKeyValue()+sep) && (sep+keysValues+sep).contains(sep+klo.getFirstKeyValue()+sep))
					toDelete.add(klo);
			}
		}
		for(int i=0;i<toDelete.size();i++)
			items.remove(toDelete.get(i));
	}
	@SuppressWarnings("unchecked")
	public boolean remove(KObject ko){
		boolean ret=items.remove(ko);
		if (ret) ko.toDelete();
		deletedItems.add((T) ko);
		return ret;
	}
	public void clear(){
		items.clear();
	}
	public int count(){
		return items.size();
	}
	public int indexOf(T ko){
		return items.indexOf(ko);
	}
	public int indexByCriteria(String criteria){
		int i=0;boolean trouve=false;
		while(i<items.size()&&!trouve){
			if(items.get(i).matchWith(criteria))
				trouve=true;
			i++;
		}
		if (trouve)
			return i-1;
		else
			return -1;
	}
	public void setSql(String sql){
		if(sql=="")
			try {
				this.sql=clazz.newInstance().getQuery();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		else
			this.sql=sql;
	}
	public String getSql() {
		return sql;
	}
	public String getUniqueId(){
		String ret="";
		try {
			ret=this.clazz+"#"+this.getSql();
			ret=ret.toLowerCase();
		} catch (Exception $e) {
		}
		return ret;
	}
	public String getUniqueIdHash(){
		return KHash.getMD5(getUniqueId());
	}
	public void loadFromDb(KDataBase db){
		loadFromDb(db, "");
	}
	public void filterFromDb(KDataBase db,String filter) throws InstantiationException, IllegalAccessException, IllegalArgumentException, NoSuchFieldException{
		String sql=KObject.makeSQLFilter_(filter, clazz);
		this.loadFromDb(db, sql);
	}
	public void loadFromDb(KDataBase db,int sortedFieldNum){
		setSql("");
		this.sql+=" ORDER BY "+Integer.toString(sortedFieldNum);
		loadFromDb(db, this.sql);
	}
	
	public void loadFromDbByFieldKeys(KDataBase db,String[] primaryKeyValues,String field){
		loadFromDbByFieldKeys(db, primaryKeyValues, field, new KDuoClasseList());
	}
	public void loadFromDbByFieldKeys(KDataBase db,String[] primaryKeyValues,String field,KDuoClasseList duoClassList){
		try {
			String where=KStrings.implode(primaryKeyValues, field, " OR ", "'", "=", KGlueMode.KEY_AND_VALUE, true);
			String sql=KObject.getQuery(clazz, where);
			loadFromDb(db,sql,duoClassList);
		} catch (InstantiationException | IllegalAccessException e) {
		}
	}
	
	public void loadFromDbByFieldKeys(KDataBase db,String[] primaryKeyValues,KConstraintHasAndBelongsToMany constraint,KDuoClasseList duoClassList){
		try {
			String where=KStrings.implode(primaryKeyValues, constraint.getFkField(), " OR ", "'", "=", KGlueMode.KEY_AND_VALUE, true);
			String sql=constraint.getSql();
			sql=KSqlQuery.addWhere(sql, where);
			loadFromDb(db,sql,duoClassList);
		} catch (InstantiationException | IllegalAccessException e) {
		}
	}
	
	public void loadFromDbByPriKeys(KDataBase db,String[] primaryKeyValues){
		try {
			String key=KObject.getFirstKey(clazz);
			loadFromDbByFieldKeys(db, primaryKeyValues, key);
		}catch(Exception e){}
	}
	
	public void loadFromDbByPriKeys(KDataBase db,String[] primaryKeyValues,KDuoClasseList duoClassList){
		try {
			String key=KObject.getFirstKey(clazz);
			loadFromDbByFieldKeys(db, primaryKeyValues, key,duoClassList);
		}catch(Exception e){}
	}
	
	public void loadFromDbByPriKeysNotIn(KDataBase db,String[] primaryKeyValues){
		try {
			String key=KObject.getFirstKey(clazz);
			String where=KStrings.implode(primaryKeyValues, key, " AND ", "'", "!=", KGlueMode.KEY_AND_VALUE, true);
			String sql=KObject.getQuery(clazz, where);
			loadFromDb(db, sql);
		} catch (InstantiationException | IllegalAccessException e) {
		}
	}
	public void loadFromDb(KDataBase db,String sql){
		loadFromDb(db, sql, new KDuoClasseList());
	}
	public void loadOne(String id) throws InstantiationException, IllegalAccessException, ClassNotFoundException, SQLException {
		loadFromDbByPriKeys(Ko.kdatabase(), new String[]{id});
	}
	@SuppressWarnings("unchecked")
	public void loadFromDb(KDataBase db,String sql,KDuoClasseList duoClasseList){
		Ko.observable.setChanged();
		Ko.fireEvent(new KFireEvent(KEvents.getEvent(this.clazz,KEventType.etLoadKlistObjectStart), this));
		setSql(sql);
		System.out.println(sql);
		KListObject<T> kl=null;
		if (Ko.useCache){
			kl=(KListObject<T>) KCache.getObjectValue(this);
			if (kl!=null)
				items=kl.asAL();
		}
		if(db!=null){
			if (kl==null){
				MapArrayString<KConstraint> membersKeys=new MapArrayString<>();
				try {
					KDbResultSet rs=db.sendQuery(this.sql);
					KDebugConsole.print(clazz.getSimpleName(), "KOBJECT", "KListObject<"+clazz.getSimpleName()+">.loadFromDb");
					boolean next=false;
					do {
						T ko=clazz.newInstance();
						next=ko.loadFromDb(rs,true);
						if (next){ 
							items.add((T) ko);
							ko.getConstraints().preload(membersKeys);
						}
					}while (next);
					if(duoClasseList.size()<Ko.ConstraintsDepth)
						loadConstraintsFromList(membersKeys,duoClasseList);
					
					if(Ko.useCache)
						KCache.put(this);
					rs.close();
				} catch (Exception e) {
					Ko.klogger().log(Level.WARNING, "Impossible de charger la liste :"+clazz, e);
				}
			}
			try {
				db.close();
			} catch (SQLException e) {
				Ko.klogger().log(Level.WARNING, "Impossible de fermer la connexion :"+db, e);
			}
		}
		Ko.observable.setChanged();
		Ko.fireEvent(new KFireEvent(KEvents.getEvent(this.clazz,KEventType.etLoadKlistObjectEnd), this));
	}
	@SuppressWarnings("unchecked")
	private void loadConstraintsFromList(MapArrayString<KConstraint> membersKeys,KDuoClasseList duoClasseList) throws InstantiationException, IllegalAccessException, ClassNotFoundException, SQLException{
		
		for(KConstraint preloadConstraint:membersKeys){
			Class<? extends KObject> clazz=preloadConstraint.getClazzList();
			Class<? extends KConstraint> constraintClazz=preloadConstraint.getClass();
			KListObject<? extends KObject> kl=new KListObject<>(clazz);
			ArrayList<KDuoClasse> tmpList=(ArrayList<KDuoClasse>) duoClasseList.getDuoClasses().clone();
			duoClasseList.add(preloadConstraint.getClazz(), this.clazz);
			if(constraintClazz.equals(KConstraintBelongsTo.class))
				kl.loadFromDbByPriKeys(Ko.kdatabase(), (membersKeys.get(preloadConstraint)+"").split(";"),duoClasseList);
			else if(constraintClazz.equals(KConstraintHasMany.class))
				kl.loadFromDbByFieldKeys(Ko.kdatabase(), (membersKeys.get(preloadConstraint)+"").split(";"),preloadConstraint.getFkField(),duoClasseList);
			else
				kl.loadFromDbByFieldKeys(Ko.kdatabase(), (membersKeys.get(preloadConstraint)+"").split(";"),preloadConstraint.getFkField(),duoClasseList);
			duoClasseList.setDuoClasses(tmpList);
			for(KObject ko:items){
				KConstraint co= ko.getConstraints().getConstraint(preloadConstraint.getMember(), preloadConstraint.getClass());
				if(co!=null)
					co.load(kl);
			}
		}
	}
	private void loadFromDbForConstraint(KDataBase db,String sql,KDuoClasseList duoClasseList){
		setSql(sql);
		if(db!=null){
			try {
				KDbResultSet rs=db.sendQuery(this.sql);
				boolean next=false;
				do {
					T ko=clazz.newInstance();
					next=ko.loadFromDbForConstraint(rs,duoClasseList);
					if (next) items.add((T) ko);
				}while (next);
			} catch (Exception e) {
				Ko.klogger().log(Level.SEVERE, "Impossible de charger la liste pour la constrainte :"+clazz, e);
			}
		}
	}	
	private void updateItemsToDelete(){
		ArrayList<T> indToDelete=new ArrayList<T>();
		for (T ko : items) {
			if(ko.getRecordStatus()==KRecordStatus.rsDelete)
				indToDelete.add(ko);
		}
		for(T ko:indToDelete)
			remove(ko);
	}
	public void saveToDb(KDataBase db){
		updateItemsToDelete();
		for (T ko : items) {
			ko.updateToDb(db,true);
		}
		for (T ko : deletedItems) {
			ko.updateToDb(db,true);
		}	
		if(Ko.useCache)
			KCache.replace(this);
	}
	@SuppressWarnings("unchecked")
	public static KListObject<? extends KObject> kload(Class<? extends KObject> clazz, KDataBase db,int sortedFieldNum){
		KListObject<? extends KObject> list=new KListObject<KObject>((Class<KObject>) clazz);
		list.loadFromDb(db, sortedFieldNum);
		return list;
	}	
	@SuppressWarnings("unchecked")
	public static KListObject<? extends KObject> kload(Class<? extends KObject> clazz, KDataBase db,String sql){
		KListObject<? extends KObject> list=new KListObject<KObject>((Class<KObject>) clazz);
		list.loadFromDb(db, sql);
		return list;
	}
	
	@SuppressWarnings("unchecked")
	public static KListObject<? extends KObject> kloadForConstraint(Class<? extends KObject> clazz, KDataBase db,String sql,KDuoClasseList duoClasseList){
		KListObject<? extends KObject> list=new KListObject<KObject>((Class<KObject>) clazz);
		list.loadFromDbForConstraint(db, sql,duoClasseList);
		return list;
	}
	
	@SuppressWarnings("unchecked")
	public static KListObject<? extends KObject> kload(Class<? extends KObject> clazz, KDataBase db){
		KListObject<? extends KObject> list=new KListObject<KObject>((Class<KObject>) clazz);
		list.loadFromDb(db);
		return list;
	}
	public String showWithMask(String mask){
		return showWithMask(mask, "{","}");
	}
	public String showWithMask(String mask,String sepFirst,String sepLast){
		String ret="";
		for (T ko : items) {
			ret+=ko.showWithMask(mask, sepFirst,sepLast)+"\n";
		}
		return ret;
	}
	public String showWithMaskGroupBy(String mask,String groupByAttribute){
		return showWithMaskGroupBy(mask, groupByAttribute, "{"+groupByAttribute+"}");
	}
	public String showWithMaskGroupBy(String mask,String groupByAttribute,String groupByMask){
		return showWithMaskGroupBy(mask, groupByAttribute, 
		"<div name='grp-"+groupByAttribute+"' id='grp-{"+groupByAttribute+"}' class='gbHeader'>"+
		groupByMask+"</div><div id='grp-child-{"+groupByAttribute+"}' class='gbChild'>",true);
	}

	public String showWithMaskGroupBy(String mask,String groupByAttribute,String attributeMask,boolean hasJs){
		this.sortBy(groupByAttribute);
		String gb="";
		String ret="";
		String js="<script type='text/javascript'>";
		boolean toClose=false;
		for (T ko : items) {
			try {
				String tmpGb=ko.getAttribute(groupByAttribute)+"";
				if(!gb.equals(tmpGb)){
						if(toClose){
							ret+="</div>";
							toClose=false;
						}
						ret+=ko.showWithMask(attributeMask);
						if(hasJs)
							js+="(new Forms.Accordion($('grp-child-"+tmpGb+"'),5,24)).attach($('grp-"+tmpGb+"'));";
						toClose=true;
						gb=tmpGb;
				}
			} catch (Exception e) {}
			ret+=ko.showWithMask(mask)+"\n";
		}
		if(toClose)
			ret+="</div>";
		if(hasJs)
			ret+=js+"</script>";
		return ret;
	}
	public String showWithMask(KMask mask){
		return mask.show(this);
	}
	public KListObject<T> select(String criteria){
		KListObject<T> list=new KListObject<T>(clazz);
		for (T ko : items) {
			if(ko.matchWith(criteria)) list.add(ko,true);
		}
		return list;
	}
	public KObject selectFirst(String criteria){
		KObject result=null;
		for (T ko : items) {
			if(ko.matchWith(criteria)) {
				result=ko;
				break;
			}
		}
		return result;
	}
	public KListObject<T> selectByName(String value){
		KListObject<T> list=new KListObject<T>(clazz);
		for (T ko : items) {
			if(ko.matchWithValue(value)) list.add(ko);
		}
		return list;
	}
	public void setRecordStatus(KRecordStatus recordStatus){
		for(KObject ko:items)
			ko.setRecordStatus(recordStatus);
	}
	public void markForHasAndBelongsToMany(String keys,String separator){
		setRecordStatus(KRecordStatus.rsDelete);
		ArrayList<String> listKeys=new ArrayList<String>(Arrays.asList(keys.split(separator)));
		for(KObject ko:items){
			String k=ko.getFirstKeyValue();
			if((separator+keys+separator).contains(separator+k+separator)){
				ko.setRecordStatus(KRecordStatus.rsNone);
				listKeys.remove(k);
				}
		}
		for(String k:listKeys){
			KObject ko;
			try {
				if(!"".equals(k)){
					ko = clazz.newInstance();
					ko.setFirstKeyValue(k);
					add(ko);
				}
			} catch (Exception e) {}
		}
	}
	public String getKeyValuesForHasAndBelongsToMany(String separator){
		String result="";
		for(KObject ko:items)
			result+=ko.getFirstKeyValue()+separator;
		return result;
	}
	public String getFieldValues(String fieldName,String separator){
		String result="";
		for(KObject ko:items)
			try {
				result+=ko.getAttribute(fieldName)+separator;
			} catch (SecurityException | NoSuchFieldException
					| IllegalArgumentException | IllegalAccessException e) {
				result=null;
			}
		return result;
	}
	public ArrayList<T> asAL(){
		return items;
	}
	public Iterator<T> iterator(){
		return items.iterator();
	}
	public void sortBy(String attribute,boolean asc){
		KQuickSort.triRapide(this, attribute,asc);
	}
	public void sortBy(String attribute){
		sortBy(attribute,true);
	}
	/**
	 * Trie la collection en ordre croissant
	 */
	public void sort(){
		sort(true);
	}
	/**
	 * Trie la collection en invoquant la méthode compareTo de chaque objet 
	 * @param asc sens, si vrai croissant
	 */
	@SuppressWarnings("unchecked")
	public void sort(boolean asc){
		if(asc==true)
			Collections.sort(items);
		else
			Collections.reverse(items);
	}
	
	public void sort(String[] attributes,boolean asc){
		if(asc==true)
			Collections.sort(items,getComparator(attributes));
		else
			Collections.sort(items,getComparatorDesc(attributes));
	}
	
	private Comparator<T> getComparator(final String[] attributes){
		return getComparator(attributes, 0);
	}
	
	private Comparator<T> getComparator(final String[] attributes,final int profondeur){
		Comparator<T> comp=new Comparator<T>() {

			@Override
			public int compare(KObject o1, KObject o2) {
				int result=0;
				try {
					result=KQuickSort.compare(o1, o2, attributes[profondeur]);
					if(result==0&&profondeur<attributes.length-1)
						result=KQuickSort.compare(o1, o2, attributes[profondeur+1]);
						
				} catch (SecurityException | IllegalArgumentException
						| NoSuchFieldException | IllegalAccessException e) {}
				return result;
			}
		};
		return comp;
	}
	private Comparator<T> getComparatorDesc(final String[] attributes){
		return getComparatorDesc(attributes, 0);
	}	
	private Comparator<T> getComparatorDesc(final String[] attributes,final int profondeur){
		Comparator<T> comp=new Comparator<T>() {

			@Override
			public int compare(KObject o1, KObject o2) {
				int result=0;
				try {
					result=KQuickSort.compare(o1, o2, attributes[profondeur]);
					if(result==0&&profondeur<attributes.length-1)
						result=KQuickSort.compare(o1, o2, attributes[profondeur+1]);
						
				} catch (SecurityException | IllegalArgumentException
						| NoSuchFieldException | IllegalAccessException e) {}
				return -result;
			}
		};
		return comp;
	}
	
	/**
	 * Retourne une arrayList des éléments de la collection
	 * @return items
	 */
	public ArrayList<T> getItems() {
		return items;
	}
	public KListObject<? extends KObject> getMemberAsKL(String member,Class<? extends KObject> clazz){
		KListObject<? extends KObject> kl=new KListObject<>(clazz);
		for(KObject ko:items){
			try {
				KObject o=(KObject) ko.getAttribute(member);
				if(o!=null)
					kl.add(o, true);
				else
					return kl;
			} catch (Exception e) {
				return kl;
			}
		}
		return kl;
	}
	/**
	 * Affecte la valeur value au membre memberName de chacun des objets de la liste
	 * @param memberName Membre à modifier
	 * @param value Valeur à affecter
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 * @throws NoSuchFieldException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	public void setMemberValue(String memberName,Object value) throws SecurityException, IllegalArgumentException, NoSuchFieldException, IllegalAccessException, InvocationTargetException{
		for(KObject o:items){
			o.setAttribute(memberName, value);
		}
	}
	
	/**
	 * Exécute une commande sur l'ensemble des objets de la liste
	 * @param commande Commande à exécuter
	 */
	public void appyCommande(IKommande commande){
		for(KObject o:items){
			commande.execute(o);
		}
	}
	
}
