/**
 * Classe KDataBase
 * 
 * @author jcheron
 * @link http://www.kobject.net/
 * @copyright Copyright kobject 2008-2010
 * @license http://www.kobject.net/index.php?option=com_content&view=article&id=50&Itemid=2  BSD License
 * @version $Id: KDataBase.java,v 1.8 2011/01/14 01:12:55 jcheron Exp $
 * @package ko.ksql
 */
package net.ko.ksql;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;

import net.ko.bean.KDbParams;
import net.ko.framework.KDebugConsole;

public abstract class KDataBase{
	protected KDbParams params;

	protected Connection connection;
	protected String driver;
	@SuppressWarnings("unused")
	private static KDataBase instance;
	private ResultSet result;
	private Statement statement;
	@SuppressWarnings("unused")
	private String query;
	protected String options="";
	protected boolean inuse;
	protected long timestamp;
	protected KDbConnectionPool pool=null;

	protected synchronized void _connect() throws ClassNotFoundException, SQLException{
	}
	public static boolean isValid(Connection connection){
	   if(connection==null){
	      return false;
	   }
	   ResultSet ping = null;
	   try{
	      if(connection.isClosed()){return false;}
	      ping = connection.createStatement().executeQuery("SELECT 1");
	      return ping.next();
	   }catch(SQLException sqle){
	      return false;
	   }
	   finally{
	      if(ping!=null){try{ping.close();}catch(Exception e){}}
	   }  
	}
	public Connection getConnection() {
		return connection;
	}
	public static boolean isValid(KDataBase db){
		if(db!=null)
			return KDataBase.isValid(db.getConnection());
		else
			return false;
	}
	public boolean isValid(){
		return KDataBase.isValid(connection);
	}
	private Statement getStatement() throws SQLException{
		if (statement==null)
			statement=connection.createStatement();
		return statement;
	}
	public KDataBase() {
		timestamp=0;
		inuse=false;
		params=new KDbParams();
	}
	
	public KDataBase(String host, String user, String pass, String base) {
		super();
		params=new KDbParams(host, user, pass);
		params.setBase(base);
	}
	public KDataBase(String host, String user, String pass, String base,String port) {
		super();
		params=new KDbParams(host, user, pass ,base ,port);
		
	}	
	public void connect(String host, String user, String pass, String base) throws ClassNotFoundException, SQLException {
		params=new KDbParams(host, user, pass ,base);
		_connect();
	}
	public void connect(String host, String user, String pass, String base,String port) throws ClassNotFoundException, SQLException{
		params=new KDbParams(host, user, pass ,base ,port);
		_connect();
	}
	public void connect(String host, String user, String pass, String base,String port,String options) throws ClassNotFoundException, SQLException{
		params=new KDbParams(host, user, pass ,base ,port);
		this.options=options;
		_connect();
	}	
	public void connect() throws ClassNotFoundException, SQLException{
		connect(params.getHost(), params.getUser(), params.getPass(), params.getBase());
	}
	public ArrayList<String> getDbNames() throws SQLException{
		ResultSet rs= connection.getMetaData().getCatalogs();
		ArrayList<String> ret=new ArrayList<String>();
		while(rs.next())
			ret.add(rs.getString("TABLE_CAT"));
		return ret;
	}
	public ArrayList<String> getTableNames(String dbName) throws SQLException{
		ResultSet tables = connection.getMetaData().getTables(dbName,null,"%",null);
		ArrayList<String> ret=new ArrayList<String>();
		while(tables.next())
			ret.add(tables.getString("TABLE_NAME"));
		return ret;
	}
	public ArrayList<String> getFieldNames(String tableName) throws SQLException{
		ResultSet rs= connection.getMetaData().getColumns(connection.getCatalog(), null, tableName, "%");
		ArrayList<String> ret=new ArrayList<String>();
		while(rs.next()){
			ret.add(rs.getString("COLUMN_NAME"));
		}
		return ret;
	}
	public String autoInc(String tableName) throws SQLException{
		String autoIncField="";
		ResultSet rs=getStatement().executeQuery(KSqlQuery.makeSelect(tableName));
		ResultSetMetaData rsmd=rs.getMetaData();
		int i=1;
		while(i<=rsmd.getColumnCount() && autoIncField.equals("")){
			if (rsmd.isAutoIncrement(i))
				autoIncField=rsmd.getColumnName(i);
			i++;
		}
		return autoIncField;
	}
	public HashMap<String,Object> getFieldNamesAndTypes(String tableName) throws SQLException{
		HashMap<String,Object> ret=new HashMap<String, Object>();
		ResultSet rs=getStatement().executeQuery(KSqlQuery.makeSelect(tableName));
		ResultSetMetaData rsmd=rs.getMetaData();
		for(int i=1;i<=rsmd.getColumnCount();i++)
			ret.put(rsmd.getColumnName(i), new KDbField(i, rsmd));
		
		ArrayList<String> keys=getKeyFields(tableName);
		for(String pkKield:keys){
			if(ret.containsKey(pkKield))
				((KDbField)ret.get(pkKield)).setPrimary(true);
		}
		/*ResultSet rs= connection.getMetaData().getColumns(connection.getCatalog(), null, tableName, "%");
		HashMap<String,Object> ret=new HashMap<String, Object>();
		while(rs.next()){
			KDbField field=new KDbField(rs);
			ret.put(rs.getString("COLUMN_NAME"),field);
		}
		String autoInc=autoInc(tableName);
		if(!autoInc.equals(""))
			((KDbField)ret.get(autoInc)).setAutoInc(true);*/
		return ret;
	}	
	public ArrayList<String> getKeyFields(String tableName) throws SQLException{
		//ResultSet clefs = connection.getMetaData().getPrimaryKeys(null,null,tableName);
		ResultSet clefs = connection.getMetaData().getIndexInfo(null,null, tableName, true,true);
		ArrayList<String> ret=new ArrayList<String>();
		while(clefs.next()){
			String str=clefs.getString("INDEX_NAME");
			if (str!=null)
				if(str.toLowerCase().startsWith("pri"))
					ret.add(clefs.getString("COLUMN_NAME"));
		}		
		return ret;
	}
	public ArrayList<KForeignKey> getForeignKeys(String tableName) throws SQLException{
		ResultSet keys = connection.getMetaData().getExportedKeys(null,null,tableName);
		ArrayList<KForeignKey> ret=new ArrayList<KForeignKey>();
		while(keys.next()){
			KForeignKey fk=new KForeignKey(keys.getString("PKTABLE_NAME"), keys.getString("PKCOLUMN_NAME"), keys.getString("FKTABLE_NAME"), keys.getString("FKCOLUMN_NAME"));
			ret.add(fk);
		}		
		return ret;
	}	
	public boolean tableExist(String tableName){
	    boolean ret=false;
		String query = "SELECT * FROM  " + tableName + " WHERE 1=0";
	    Statement stmt = null;
	    try {
	      stmt = connection.createStatement();
	      stmt.executeQuery(query);
	      ret=true;
	    }
	    catch (Exception e ) {
	      ret=false;
	    }
	    return ret;
	}
	public static  Connection getConnexion(String host, String user, String pass, String base){return null;}

	public KDbResultSet sendQuery(String sql) throws SQLException{
		ResultSet resultset=getStatement().executeQuery(sql);
		KDebugConsole.print(sql, "SQL","KDataBase.sendQuery");
		result=resultset;
		return new KDbResultSet(resultset,this);
	}
	
	public Object getValue(String sql) throws SQLException{
		Object o=null;
		ResultSet resultset=getStatement().executeQuery(sql);
		result=resultset;
		if(resultset.next()){
			o=resultset.getObject(1);
		}
		resultset.close();
		return o;
	}
	public ArrayList<String> insert_id() throws SQLException{
		ResultSet resultSet = getStatement().getGeneratedKeys(); 
		ArrayList<String> result=new ArrayList<String>();
		if ( resultSet != null && resultSet.next() ) {
			ResultSetMetaData rsMetaData = resultSet.getMetaData();
			int columnCount = rsMetaData.getColumnCount();
			for (int i = 1; i <= columnCount; i++) {
				result.add(resultSet.getString(i));
			}
		}
		return result;
	}
	public int num_rows() throws SQLException {
		int res=0;
		if (result!=null){
			result.last();
			res=result.getRow();
		}
		return res;
		
	}
	public int rowCount(String sql) throws SQLException{
		result=getStatement().executeQuery(sql);
		result.last();
		return result.getRow();
	}
	public KDbParams getParams(){
		return params;
	}
	public String getBase(){
		if(connection!=null)
			try {
				return connection.getCatalog();
			} catch (SQLException e) {}
		return "";
	}
	public void close() throws SQLException{
		if(pool!=null){
			pool.returnConnection(this);
			KDebugConsole.print(this+" dispo","POOL","KDataBase.close");}
		else
			if(connection!=null){
				if(statement!=null)
					statement.close();
				connection.close();
			}
	}
	public int execute(String sql) throws SQLException{
		KDebugConsole.print(sql, "SQL","KDataBase.execute");
		return getStatement().executeUpdate(sql,Statement.RETURN_GENERATED_KEYS);
	}
	public HashMap<String, Object> fetchMap() throws SQLException{
		if (result.next()){
			return new KDbResultSet(result).toMap();}
		return null;
	}
	public String getOptions() {
		return options;
	}
	public void setOptions(String options) {
		this.options = options;
	}
	public String getBaseName(){
		return params.getBase();
	}
    public synchronized boolean lease() {
        if(inuse)  {
            return false;
        } else {
           inuse=true;
           KDebugConsole.print(this+" inuse","POOL","KDataBase.lease");
           timestamp=System.currentTimeMillis();
           return true;
        }
     }
    public boolean inUse() {
        return inuse;
    }

    public long getLastUse() {
        return timestamp;
    }

    public void expireLease() {
        inuse=false;
        KDebugConsole.print(this+" expirelease","POOL","KDataBase.expireLease");
    }
	public void setPool(KDbConnectionPool pool) {
		this.pool = pool;
	}
	public abstract String getURL();

	public void clear(){
			try {
				if(result!=null)
					result.close();
				if(statement!=null)
					statement.close();
				if(connection!=null)
					connection.close();
			} catch (SQLException e) {
			}
	}
}

