package net.ko.ksql;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.logging.Level;

import net.ko.framework.KDebugConsole;
import net.ko.framework.Ko;

public class KDbConnectionPool{
	private ArrayList<KDataBase> connections;
	private static KDbConnectionPool instance;
	final private long timeout=60000;
	private KDataBaseConnectionReaper reaper;
	KDbConnectionPool(){
		connections=new ArrayList<KDataBase>();
		reaper=new KDataBaseConnectionReaper(this);
		reaper.start();
	}
	public static KDbConnectionPool getInstance(){
		if(instance==null)
			instance=new KDbConnectionPool();
		return instance;
	}
	public static KDataBase kconnection() throws InstantiationException, IllegalAccessException{
		return getInstance().getConnection();
	}
	public void add(KDataBase connection){
		connections.add(connection);
		connection.setPool(this);
	}
	public synchronized KDataBase getConnection() throws InstantiationException, IllegalAccessException{
		KDataBase result=null;
		for(KDataBase db:connections){
			if(db.lease()){
				if(db.isValid())
					return db;
				else
					remove(db);
			}
		}
		
		result=Ko.kcreateDbInstance();
		result.lease();
		add(result);
		KDebugConsole.print("nb connections: "+connections.size(),"POOL","KDataBase.getConnection");
		return result;
	}
	public void remove(KDataBase db){
		try {
			db.close();
		} catch (SQLException e) {
			Ko.klogger().log(Level.SEVERE, "Impossible de fermer la connexion à la base de données", e);
		}
		connections.remove(db);
	}
	public synchronized void returnConnection(KDataBase db) {
		db.expireLease();
	}
	public synchronized void reapConnections() {
		long stale=System.currentTimeMillis()-timeout;
		ArrayList<KDataBase> toRemove=new ArrayList<KDataBase>();
		if(connections!=null){
			for(KDataBase db:connections){
				if(db.inUse()&&stale>db.getLastUse()&&!KDataBase.isValid(db)){
					KDebugConsole.print("remove "+db,"POOL","KDbConnectionPool.reapConnections");
					toRemove.add(db);
				}
			}
			for(KDataBase db:toRemove)
				remove(db);
		}
	}
	private synchronized void _removeAll(){
		ArrayList<KDataBase> toRemove=new ArrayList<KDataBase>();
		for(KDataBase db:connections)
			toRemove.add(db);
		for(KDataBase db:toRemove){
			try {
				db.close();
				db.clear();
			} catch (SQLException e) {}
		}
	}
	public static void removeAll(){
		if(instance!=null)
			instance._removeAll();
	}
	public static void setToNull(){
		instance=null;
	}
}
