package net.ko.mapping;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import net.ko.types.KStringsType;
import net.ko.utils.KString;
import net.ko.utils.KStrings;

public class KAjaxInclude extends KAjaxWithChilds implements IHasURL{
	protected String method;
	protected String targetId;
	protected String targetURL;
	protected String targetParams;
	protected String formName;
	protected String condition;
	protected KStrings parameters;
	public KAjaxInclude(){
		super();
		this.condition="";
	}
	protected KAjaxInclude(String targetId,String targetURL){
		this(targetId, targetURL, "get", "", "","");
	}
	public KAjaxInclude(String targetId,String targetURL,String method,String targetParams,String targetFunction,String formName){
		this();
		this.targetId=targetId;
		this.targetURL=targetURL;
		setMethod(method);
		this.targetParams=targetParams.replace(",", "&");
		this.targetFunction=targetFunction;
		this.formName=formName;
		parameters=new KStrings(this.targetParams,KStringsType.kstQueryString);
	}
	public String getMethod() {
		return method;
	}

	public void setMethod(String method) {
		if(method==null||"".equals(method))
			method="get";
		this.method = method;
	}
	public String getTargetId() {
		return targetId;
	}
	public void setTargetId(String targetId) {
		this.targetId = targetId;
	}
	public String getTargetURL() {
		return targetURL;
	}
	public void setTargetURL(String targetURL) {
		this.targetURL = targetURL;
	}
	public String getTargetParams() {
		return targetParams;
	}
	public void setTargetParams(String targetParams) {
		this.targetParams = targetParams;
		parameters=new KStrings(this.targetParams,KStringsType.kstQueryString);
	}
	public void addParameter(String name,String value){
		parameters.put(name, value);
	}
	public String getTargetFunction() {
		return targetFunction;
	}
	public void setTargetFunction(String targetFunction) {
		this.targetFunction = targetFunction;
	}
	@Override
	public boolean isValid(){
		boolean result=true;
		result=result&&KString.isNotNull(targetId);
		result=result&&KString.isNotNull(targetURL);
		return result;
	}
	public String createParameters(){
		String paramStr=parameters.implode();
		paramStr=replaceJsExpressions(paramStr);
		String sep="";
		if(!"".equals(paramStr))
			sep="&";
		paramStr="'"+paramStr+sep+"'+$qs(this)";
		return paramStr;
	}
	@Override
	public String createFunction(){
		String paramStr=createParameters();
		String start="(";
		if(isChild)
			start="WR(obj,";
		String result="$"+method.toLowerCase()+start+"'"+targetId+"','"+targetURL+"',"+paramStr;
		if(formName!=null&&!"".equals(formName))
			result="$"+method.toLowerCase()+"Form"+start+"'"+formName+"','"+targetId+"','"+targetURL+"',"+paramStr;
		String functionStr=getFunctions();
		if(!"".equals(functionStr))
			result+=","+functionStr;
		result= result+");";
		if(condition!=null&&!"".equals(condition))
			result="if("+condition+"){"+result+"}";
		return result;
	}
	@Override
	public String toString() {
		String result=createFunction();
		if (isChild)
			result="function(obj){"+result+"}";
		return result;
	}
	@Override
	public void initFromXml(Node item) {
		Element e=(Element)item;
		this.targetId=e.getAttribute("targetId");
		this.targetURL=e.getAttribute("targetURL");
		setMethod(e.getAttribute("method"));
		String targetParams=e.getAttribute("targetParams");
		if(targetParams!=null)
			this.targetParams=targetParams.replace(",", "&");
		this.condition=e.getAttribute("condition");
		this.targetFunction=e.getAttribute("targetFunction");
		this.formName=e.getAttribute("formName");
		parameters=new KStrings(this.targetParams,KStringsType.kstQueryString);
		childs=KAjaxJs.getChilds(item,true);
	}
	@Override
	public String getURL() {
		return targetURL;
	}
	
	@Override
	public String getDisplayCaption() {
		return targetId;
	}
	@Override
	public String getDisplayStructure() {
		String postFrm="";
		String getParams="()";
		if(formName!=null&&!"".equals(formName))
			postFrm="["+formName+"]";
		if(targetParams!=null&&!"".equals(targetParams))
			getParams="("+targetParams+")";
		return "<fieldset class='targetId'><legend>{caption}</legend><span class='targetUrl'>"+targetURL+"</span>-><span class='method'>"+getMethod()+postFrm+getParams+"</span></fieldset>";
	}
	public String getFormName() {
		return formName;
	}
}
