package net.ko.mapping;

import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import net.ko.utils.KHash;
import net.ko.utils.KString;

public class KAjaxJs implements IAjaxObject{
	private String triggerId;
	private String triggerName;
	private String triggerTagName;
	private String triggerEvent;
	private String triggerClassName;
	private ArrayList<IAjaxObject> childs;
	private HttpServletRequest request;
	private int keyCode=-1;
	
	public KAjaxJs(){
		childs=new ArrayList<>();
	}
	public KAjaxJs(String triggerId,String triggerName,String triggerTagName,String triggerEvent){
		this();
		this.triggerId=triggerId;
		this.triggerName=triggerName;
		this.triggerTagName=triggerTagName;
		setTriggerEvent(triggerEvent);
	}
	
	@Override
	public boolean isValid(){
		boolean result=true;
		result=result&&(KString.isNotNull(triggerId)||KString.isNotNull(triggerName))||(KString.isNotNull(triggerId)&&KString.isNotNull(triggerTagName));
		return result;
	}
	
	@Override
	public String createFunction(){
		String result="function(obj){";
		for(IAjaxObject obj:childs){
			if(obj instanceof KAjaxSet)
				((KAjaxSet) obj).setRequest(request);
			result+=obj.createFunction();
		}
		return result+"}";
	}
	public String getExecuteString(){
		String result="";
		for(IAjaxObject obj:childs){
			if(obj instanceof KAjaxSet)
				((KAjaxSet) obj).setRequest(request);
			result+=obj.createFunction();
		}
		return result;
	}
	
	@Override
	public String toString() {
		String result="";
		if(KString.isNotNull(triggerId)){
			if(triggerId.equalsIgnoreCase("body")&&triggerEvent.contains("load"))
				result=getExecuteString();
			else{
				if(KString.isNotNull(triggerClassName)){
					result="$addEvtByCN('"+triggerClassName+"','"+triggerEvent+"',"+createFunction()+","+getTriggerTagName()+","+getTriggerId()+");\n";
				}else if(KString.isNotNull(triggerTagName))
					result="$addEvtByTN('"+triggerId+"','"+triggerTagName+"','"+triggerEvent+"',"+createFunction()+",false);\n";
				else{
					if(keyCode<1)
						result="$addEvt($('"+triggerId+"'),'"+triggerEvent+"',"+createFunction()+",false);\n";
					else
						result="$addEvt($('"+triggerId+"'),'"+triggerEvent+"',"+createFunction()+",false,"+keyCode+");\n";
				}
			}
		}
		else if(KString.isNotNull(triggerName))
			result="$addEvts('"+triggerName+"','"+triggerEvent+"',"+createFunction()+",false);\n";
		return result;
	}
	
	public void setTriggerId(String triggerId) {
		this.triggerId = triggerId;
	}
	
	public String getTriggerName() {
		return triggerName;
	}
	
	public void setTriggerName(String triggerName) {
		this.triggerName = triggerName;
	}
	
	public String getTriggerEvent() {
		return triggerEvent;
	}
	
	protected void setTriggerEvent(String triggerEvent) {
		if(KString.isNull(triggerEvent))
			triggerEvent="click";
		if(triggerEvent.startsWith("on"))
			triggerEvent=triggerEvent.substring(2);
		triggerEvent=triggerEvent.toLowerCase();
		this.triggerEvent = triggerEvent;
	}

	public static ArrayList<IAjaxObject> getChilds(Node item,boolean isChild){
		ArrayList<IAjaxObject> result=new ArrayList<>();
		NodeList cNodes=item.getChildNodes();
		for(int i=0;i<cNodes.getLength();i++){
			KAjaxObject obj=null;
			switch (cNodes.item(i).getNodeName()) {
			case "include":
				obj=new KAjaxInclude();
				obj.initFromXml(cNodes.item(i));
				break;
			case "function":
				obj=new KAjaxFunction();
				obj.initFromXml(cNodes.item(i));
				break;
			case "message":
				obj=new KAjaxMessage();
				obj.initFromXml(cNodes.item(i));
				break;
			case "showHide":
				obj=new KAjaxShowHide();
				obj.initFromXml(cNodes.item(i));
				break;
			case "selector":
				obj=new KAjaxSelector();
				obj.initFromXml(cNodes.item(i));
				break;
			case "set":
				obj=new KAjaxSet();
				obj.initFromXml(cNodes.item(i));
				break;
			case "fireEvent":
				obj=new KAjaxEvent();
				obj.initFromXml(cNodes.item(i));
				break;
			case "messageDialog":
				obj=new KAjaxMessageDialog();
				obj.initFromXml(cNodes.item(i));
				break;
			case "button":
				obj=new KAjaxDialogButton();
				obj.initFromXml(cNodes.item(i));
				break;
			case "includeDialog":
				obj=new KAjaxIncludeDialog();
				obj.initFromXml(cNodes.item(i));
				break;
			default:
				break;
			}
			if(obj!=null&&obj.isValid()){
				obj.setChild(isChild);
				result.add(obj);
			}
		}
		return result;
	}
	@Override
	public void initFromXml(Node item) {
		Element e=(Element)item;
		setTriggerEvent(e.getAttribute("triggerEvent"));
		String triggerId=e.getAttribute("triggerId");
		if(triggerId!=null&&!"".equals(triggerId))
			this.triggerId=triggerId;
		String triggerTagName=e.getAttribute("triggerTagName");
		if(triggerTagName!=null&&!"".equals(triggerTagName))
			this.triggerTagName=triggerTagName;
		String triggerClassName=e.getAttribute("triggerClassName");
		if(triggerClassName!=null&&!"".equals(triggerClassName))
			this.triggerClassName=triggerClassName;
		String triggerName=e.getAttribute("triggerName");
		if(triggerName!=null&&!"".equals(triggerName))
			this.triggerName=triggerName;
		String strKeyCode=e.getAttribute("keyCode");
		if(strKeyCode!=null&&!"".equals(strKeyCode)){
			try{
				this.keyCode=Integer.valueOf(strKeyCode);
			}catch(Exception ex){keyCode=-1;}
		}
		childs=getChilds(item,false);
	}
	@Override
	public String execute(HttpServletRequest request) {
		this.request=request;
		return toString();
	}
	public String getTriggerTagName() {
		if(KString.isNotNull(triggerTagName))
			return "'"+triggerTagName+"'";
		else
			return "undefined";
	}
	public void setTriggerTagName(String triggerTagName) {
		this.triggerTagName = triggerTagName;
	}
	public String getTriggerClassName() {
		if(KString.isNotNull(triggerClassName))
			return "'"+triggerClassName+"'";
		else
			return "undefined";
	}
	public void setTriggerClassName(String triggerClassName) {
		this.triggerClassName = triggerClassName;
	}
	
	public String getTriggerId() {
		if(KString.isNotNull(triggerId))
			return "'"+triggerId+"'";
		else
			return "undefined";
	}

	public ArrayList<IAjaxObject> getChilds() {
		return childs;
	}
	@Override
	public String getDisplayCaption() {
		return triggerId+".on"+KString.capitalizeFirstLetter(getTriggerEvent());
	}
	
	@Override
	public String display(int margin){
		String structure=getDisplayStructure();
		structure=structure.replace("{caption}", getDisplayCaption());
		return "<div style='margin-left:"+(20*margin)+"px;' class='"+getClass().getSimpleName()+"'>"+structure+"</div>";	}
	@Override
	public String getUniqueIdHash(int niveau) {
		return KHash.getMD5(niveau+getDisplayCaption());
	}
	@Override
	public String getDisplayStructure() {
		return "{caption}";
	}
	@Override
	public boolean hasChilds() {
		return true;
	}
	@Override
	public String replaceJsExpressions(String input) {
		return input;
	}
}
