package net.ko.mapping;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.ko.framework.KDebugConsole;
import net.ko.framework.Ko;
import net.ko.http.objects.KRequest;
import net.ko.utils.KString;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class KMapping extends KFilterMapping{
	private String method;
	private boolean mainControl;
	private String classControl;
	private HashMap<String, String> parameters;
	public KMapping(){
		this("","","",false,"","");
	}
	public KMapping(String method,String requestURL,String responseURL,boolean mainControl,String classControl,String queryString){
		super(requestURL,responseURL);
		this.method=method.toUpperCase();
		this.mainControl=mainControl;
		this.classControl=classControl;
		this.parameters=new HashMap<>();
		this.queryString=queryString;
	}
	public void addParameter(String name,String value){
		parameters.put(name, value);
	}
	public String getMethod() {
		return method;
	}
	public void setMethod(String method) {
		this.method = method;
	}
	public boolean isMainControl() {
		return mainControl;
	}
	public void setMainControl(boolean mainControl) {
		this.mainControl = mainControl;
	}
	public String getClassControl() {
		return classControl;
	}
	public void setClassControl(String classControl) {
		this.classControl = classControl;
	}
	public HashMap<String, String> getParameters() {
		return parameters;
	}
	@Override
	public void initFromXml(Node item) {
		String method=((Element)item).getAttribute("method");
		if(method!=null&&!"*".equals(method))
			this.method=method;
		
		String requestURL=((Element)item).getAttribute("requestURL");
		if(requestURL!=null)
			this.requestURL=requestURL;
		
		String responseURL=((Element)item).getAttribute("responseURL");
		if(responseURL!=null)
			this.responseURL=responseURL;
		
		boolean mainControl=KString.isBooleanTrue(((Element)item).getAttribute("mainControl"));
		this.mainControl=mainControl;
		
		String classControl=((Element)item).getAttribute("classControl");
		if(classControl!=null)
			this.classControl=classControl;
		
		String queryString=((Element)item).getAttribute("queryString");
		if(queryString!=null)
			this.queryString=queryString.replace(",", "&");
		
		NodeList paramsList=item.getChildNodes();
		for(int i=0;i<paramsList.getLength();i++){
			if(paramsList.item(i).getNodeName().equalsIgnoreCase("parameter")){
				Element e=(Element)paramsList.item(i);
				String name=e.getAttribute("name");
				String value=e.getAttribute("value");
				if(name!=null &!"".equals(name))
					parameters.put(name, value);
			}
		}
		
	}

	public boolean methodMatches(HttpServletRequest request){
		boolean result=true;
		if(method!=null&&!"".equals(method)&&!"*".equals(method)){
			result=method.equalsIgnoreCase(request.getMethod());
		}
		return result;
	}
	private boolean paramMatches(String name,String value,HttpServletRequest request){
		boolean result=true;
		result=request.getParameterMap().containsKey(name);
		if(result && value!=null && !"".equals(value))
			result=value.equals(request.getParameter(name));
		return result;
	}
	public boolean paramsMatches(HttpServletRequest request){
		boolean result=true;
		if(parameters.size()>0){
			for(Map.Entry<String, String> e:parameters.entrySet()){
				String name=e.getKey();
				String value=e.getValue();
				result=paramMatches(name, value, request);
				if(!result)
					break;
			}
		}
		return result;
	}
	
	@Override
	public boolean matches(HttpServletRequest request){
		boolean result=methodMatches(request)&&requestURLMatches(request)&&paramsMatches(request);
		return result;
	}
	@SuppressWarnings("unchecked")
	private IMappingControl getSelfControl(){
		IMappingControl result=null;
		try{
			Class<IMappingControl> clazz=(Class<IMappingControl>)Class.forName(classControl);
			result=clazz.newInstance();
		}catch(Exception e){}
		return result;
	}
	public IMappingControl getMappingControl(){
		IMappingControl result=null;
		if(mainControl)
			result=Ko.kcontrolInstance();
		if(classControl!=null&&!"".equals(classControl)){
			result=getSelfControl();
		}
		return result;
	}
	@Override
	public boolean execute(HttpServletRequest request,HttpServletResponse response){
		boolean result=true;
		IMappingControl mc=getMappingControl();
		if(mc!=null){
			boolean cont=mc.beforeProcessAction(request, response);
			if(cont){
				if(mc.isValid(request, response))
					result=process(request, response);
				else{
					KDebugConsole.print("map:"+request.getServletPath()+"->invalidControl","MAPPING","KMapping.execute");
					request.removeAttribute("ajaxIncludes");
					mc.onInvalidControl(request, response);
				}
			}
		}else
			result=process(request, response);
		return result;
	}
	private String getResponseURL(HttpServletRequest request){
		String result=responseURL;
		if(responseURL.contains("$")){
			Pattern p = Pattern.compile(getRequestRegExpr(),0);
			Matcher m = p.matcher(request.getServletPath());
			result= m.replaceAll(responseURL);
		}
		return result;
	}
	private boolean process(HttpServletRequest request,HttpServletResponse response){
		boolean result=true;
		String responseURL=getResponseURL(request);
		try {
			if(queryString!=null&&!"".equals(queryString)){
				KDebugConsole.print("map:"+request.getServletPath()+"->"+responseURL+"?"+queryString,"MAPPING","KMapping.process");
				KRequest.forward(responseURL, request, response, queryString);
			}
			else{
				KDebugConsole.print("map:"+request.getServletPath()+"->"+responseURL,"MAPPING","KMapping.process");
				request.getRequestDispatcher(responseURL).forward(request,response);
			}
		} catch (ServletException | IOException e) {
			result=false;
		}
		return result;
	}
	@Override
	public String display(int niveau) {
		// TODO Auto-generated method stub
		return null;
	}
}
