package net.ko.utils;

import java.util.*;
import java.lang.reflect.*;

import net.ko.converters.DateFormatter;

public class Converter {
    @SuppressWarnings("rawtypes")
	private static Map<Class,Class> primitiveMap = new HashMap<Class,Class>();
    static {
	primitiveMap.put(boolean.class, Boolean.class);
	primitiveMap.put(byte.class, Byte.class);
	primitiveMap.put(char.class, Character.class);
	primitiveMap.put(short.class, Short.class);
	primitiveMap.put(int.class, Integer.class);
	primitiveMap.put(long.class, Long.class);
	primitiveMap.put(float.class, Float.class);
	primitiveMap.put(double.class, Double.class);
    }

    public static Object convert(String value, Class<?> destClass) {
    	if ((value == null) || "".equals(value)) {
    		return value;
    	}

    	if (destClass.isPrimitive()) {
    		destClass = primitiveMap.get(destClass);
    	}

    	try {
    		Method m = destClass.getMethod("valueOf", String.class);
    		int mods = m.getModifiers();
    		if (Modifier.isStatic(mods) && Modifier.isPublic(mods)) {
    			m.setAccessible(true);
    			return m.invoke(null, value);
    		}
    	}
    	catch (NoSuchMethodException e) {
    		if (destClass == Character.class) {
    			return Character.valueOf(value.charAt(0));
    		}
    		if (destClass == java.sql.Time.class){
    			return DateFormatter.getKoTime(value);
    		}
    	}
    	catch (IllegalAccessException e) {
    	}
    	catch (InvocationTargetException e) {
    	}

	return value;
    }
}